@@ -10,6 +10,7 @@ from __future__ import annotations
 from ascend.auth.auth import AwsV4Auth, BearerAuth, RefreshAuth
 
 import requests
+from retry import retry
 import urllib3
 
 
@@ -29,6 +30,7 @@ class Session:
       verify the server's SSL certificate
       (default is `True`)
   """
+  @retry(tries=3)
   def __init__(self, environment_hostname, access_key, secret_key, verify_ssl=True):
     if not access_key:
       raise ValueError("Missing api access key")
