@@ -464,50 +464,65 @@ class ComponentApplier:
   def _create_component(self, data_service_id, dataflow_id, component: Component):
     proto = component.to_proto(self.id_map)
     api_method = None
+    default = None
     if isinstance(component, ReadConnector):
       glog.info(f"Create ReadConnector: (ds={data_service_id} df={dataflow_id} {component.id})")
-      if not self.dry_run:
-        api_method = self.client.create_read_connector
-      else:
-        return api_pb2.ReadConnector()
+      api_method = self.client.create_read_connector
+      default = api_pb2.ReadConnector()
     elif isinstance(component, WriteConnector):
       glog.info(f"Create WriteConnector: (ds={data_service_id} df={dataflow_id} {component.id})")
-      if not self.dry_run:
-        api_method = self.client.create_write_connector
-      else:
-        return api_pb2.WriteConnector()
+      api_method = self.client.create_write_connector
+      default = api_pb2.WriteConnector()
     elif isinstance(component, Transform):
       glog.info(f"Create Transform: (ds={data_service_id} df={dataflow_id} {component.id})")
-      if not self.dry_run:
-        api_method = self.client.create_transform
-      else:
-        return api_pb2.Transform()
-    if api_method is not None:
+      api_method = self.client.create_transform
+      default = api_pb2.Transform()
+    if api_method is None:
+      raise TypeError(f"Unhandled component type: {type(component)}")
+    if not self.dry_run:
       return _tracked_call(api_method, data_service_id, dataflow_id, proto).data
+    else:
+      return default
+
+  def _delete_component(self, data_service_id, dataflow_id, component_id: str):
+    api_method = None
+    component_type = self.id_map[component_id].type
+    if component_type == 'source':
+      glog.info(f"Delete ReadConnector: (ds={data_service_id} df={dataflow_id} {component_id})")
+      api_method = self.client.delete_read_connector
+    elif component_type == 'sink':
+      glog.info(f"Delete WriteConnector: (ds={data_service_id} df={dataflow_id} {component_id})")
+      api_method = self.client.delete_write_connector
+    elif component_type == 'view':
+      glog.info(f"Delete Transform: (ds={data_service_id} df={dataflow_id} {component_id})")
+      api_method = self.client.delete_transform
+    if api_method is None:
+      raise TypeError(f"Unhandled component type: {component_type}")
+    if not self.dry_run:
+      return _tracked_call(api_method, data_service_id, dataflow_id, component_id).data
 
   def _update_component(self, data_service_id, dataflow_id, component: Component):
     proto = component.to_proto(self.id_map)
     api_method = None
+    default = None
     if isinstance(component, ReadConnector):
       glog.info(f"Update ReadConnector: (ds={data_service_id} df={dataflow_id} {component.id})")
-      if not self.dry_run:
-        api_method = self.client.update_read_connector
-      else:
-        return api_pb2.ReadConnector()
+      api_method = self.client.update_read_connector
+      default = api_pb2.ReadConnector()
     elif isinstance(component, WriteConnector):
       glog.info(f"Update WriteConnector: (ds={data_service_id} df={dataflow_id} {component.id})")
-      if not self.dry_run:
-        api_method = self.client.update_write_connector
-      else:
-        return api_pb2.WriteConnector()
+      api_method = self.client.update_write_connector
+      default = api_pb2.WriteConnector()
     elif isinstance(component, Transform):
       glog.info(f"Update Transform: (ds={data_service_id} df={dataflow_id} {component.id})")
-      if not self.dry_run:
-        api_method = self.client.update_transform
-      else:
-        return api_pb2.Transform()
-    if api_method is not None:
+      api_method = self.client.update_transform
+      default = api_pb2.Transform()
+    if api_method is None:
+      raise TypeError(f"Unhandled component type: {type(component)}")
+    if not self.dry_run:
       return _tracked_call(api_method, data_service_id, dataflow_id, component.id, proto).data
+    else:
+      return default
 
   def apply(self, data_service_id, dataflow_id, component: Component):
     """
@@ -521,6 +536,8 @@ class ComponentApplier:
     if self._component_exists(data_service_id, dataflow_id, component):
       return self._update_component(data_service_id, dataflow_id, component)
     else:
+      if component.id in self.id_map:
+        self._delete_component(data_service_id, dataflow_id, component.id)
       return self._create_component(data_service_id, dataflow_id, component)
 
   @staticmethod
